@@ -7,16 +7,15 @@ from __future__ import annotations
 from functools import partial
 from typing import List
 from vsutil import get_depth, get_y, scale_value, EXPR_VARS
-from .util import _morpho_matrix, plane_stats
+from .util import _morpho_matrix
 import vapoursynth as vs
 core = vs.core
 
 
 def retinex(src: vs.VideoNode,
             sigmas: List[float | int] = [25, 80, 250],
-            lower_thr: float = 0,
-            upper_thr: float = 0,
-            cuda: bool = False,
+            lower_thr: float = 0.001,
+            upper_thr: float = 0.001,
             fast: bool = False
             ) -> vs.VideoNode:
     """
@@ -24,16 +23,12 @@ def retinex(src: vs.VideoNode,
 
     :param src: Input clip.
     :param sigmas: Sigma list for Gaussian blur.
-    :param lower_thr: Controls the white balance, will be VERY SLOW if lower_thr > 0 and cuda=False.
-    :param upper_thr: Controls the white balance, will be VERY SLOW if upper_thr > 0 and cuda=False.
-    :param cuda: Enables cupy, will be VERY SLOW if False and lower_thr > 0 or upper_thr > 0.
-    :param fast: Replaces the strongest Gaussian blur with PlaneStatsAverage.
+    :param lower_thr: Controls the white balance.
+    :param upper_thr: Controls the white balance.
+    :param fast: Replaces the strongest Gaussian blur with PlaneStatsMax.
     """
 
-    from warnings import warn
     from vsrgtools import gauss_blur
-    if not cuda and (lower_thr > 0 or upper_thr > 0):
-        warn("retinex: you are using lower_thr/upper_thr without cuda, speed drastically reduced!")
 
     luma = get_y(src).std.PlaneStats()
     is_float = luma.format.sample_type == vs.FLOAT
@@ -55,7 +50,7 @@ def retinex(src: vs.VideoNode,
     expr_msr = expr_msr + f"{'+ ' * (slen-1)}log {slen} /"
     blur_list = [gauss_blur(luma_float, i) for i in sigmas]
     msr = core.akarin.Expr([luma_float] + blur_list, expr_msr)
-    msr_stats = plane_stats(msr, lower_thr, upper_thr, cuda)
+    msr_stats = msr.psm.PlaneStatsMod(lower_thr, upper_thr)
     expr_balance = "x x.PlaneStatsMin - x.PlaneStatsMax x.PlaneStatsMin - /"
 
     if not is_float:
